#!/usr/bin/env python

import os
import sys
from pymacaron.log import pymlogger
import json
from flask import Flask

sys.path.append(os.path.join(os.path.dirname(os.path.abspath(__file__)), '..'))

from pymacaron import API, letsgo


log = pymlogger(__name__)


app = Flask(__name__)


def test_crash_reporter(msg, body):

    tmpdir = '/tmp/test-pym-microservice'
    try:
        os.stat(tmpdir)
    except Exception:
        os.mkdir(tmpdir)

    data = {
        'title': msg,
        'body': json.loads(body),
    }

    log.info("Storing crash report into %s/error_report.json" % tmpdir)
    with open(os.path.join(tmpdir, "error_report.json"), "a+") as f:
        f.write(json.dumps(data))


def start(port, debug):

    api = API(
        app,
        port=8765,
        debug=False,
        error_reporter=test_crash_reporter,
    )
    api.load_apis('.', include_crash_api=True)
    api.start(serve="crash")

letsgo(__name__, callback=start)



from pymacaron.log import pymlogger


log = pymlogger(__name__)


#
# This module contains all PyMacaron models generated from the OpenAPI schema
# They are dynamically injected into pymacaron.models's global namespace when
# the OpenAPI specification is loaded.
#
# Models can therefore be imported by name, as follows:
#
# from pymacaron.models import ModelName
#



import logging
import sys

DEFAULT_LEVEL = logging.DEBUG

root = logging.getLogger()

class ContextFilter(logging.Filter):
    """Add the viewer's user_id (if defined in JWT token) to the log string"""
    def filter(self, record):
        if not hasattr(record, 'USER_ID'):
            record.USER_ID = ''
        return True

def setup_logger(celery=False):
    global root

    # Celery setups a default handler: remove it
    root.handlers = []

    ch = logging.StreamHandler(sys.stdout)

    name = 'WORKER' if celery else 'FLASK'

    FORMAT = '%(asctime)s - ' + name + ' %(process)d%(USER_ID)s %(name)s - %(levelname)s - %(message)s'
    formatter = logging.Formatter(FORMAT)

    ch.setFormatter(formatter)
    ch.addFilter(ContextFilter())
    root.addHandler(ch)
    root.setLevel(DEFAULT_LEVEL)

    # NOTE: 2020-06-22 disabled since supervisord should handle it...
    # # If setting up celery logger, also log to a file (for debugging purpose)
    # if celery:
    #     fh = logging.FileHandler('/var/log/celery-workers.log')
    #     fh.setFormatter(formatter)
    #     root.addHandler(fh)

    # Make an exception for boto and bravado: its debug level is just too verbose...
    logging.getLogger('boto').setLevel(logging.INFO)
    logging.getLogger('boto3').setLevel(logging.WARNING)
    logging.getLogger('botocore').setLevel(logging.WARNING)
    logging.getLogger('bravado_core.spec').setLevel(logging.INFO)
    logging.getLogger('bravado_core.model').setLevel(logging.INFO)
    logging.getLogger('bravado_core.operation').setLevel(logging.INFO)
    logging.getLogger('swagger_spec_validator.ref_validators').setLevel(logging.INFO)
    logging.getLogger('celery').setLevel(logging.INFO)
    logging.getLogger('scout_apm.core').setLevel(logging.INFO)
    logging.getLogger('elasticsearch').setLevel(logging.INFO)


setup_logger()

def get_logger():
    global root
    return root

def set_level(newlevel):
    global root
    root.setLevel(newlevel)


#
# A custom wrapper around the logger object, injecting user_id and call_id from
# the flask context
#

class PymacaronLogger():

    def __init__(self, name=None):
        self.logger = logging.getLogger(name)

    def get_extra(self, extra):
        from pymacaron.auth import get_userid
        s = get_userid()
        extra.update({
            'USER_ID': ' [%s]' % s if s else '',
        })
        return extra

    def error(self, s, extra={}, **kwargs):
        self.logger.error(s, extra=self.get_extra(extra), **kwargs)

    def info(self, s, extra={}, **kwargs):
        self.logger.info(s, extra=self.get_extra(extra), **kwargs)

    def warn(self, s, extra={}, **kwargs):
        self.logger.warn(s, extra=self.get_extra(extra), **kwargs)

    def warning(self, s, extra={}, **kwargs):
        self.logger.warning(s, extra=self.get_extra(extra), **kwargs)

    def debug(self, s, extra={}, **kwargs):
        self.logger.debug(s, extra=self.get_extra(extra), **kwargs)


def pymlogger(name=None):
    if not name:
        name = __name__
    return PymacaronLogger(name)



from pymacaron.log import pymlogger
import multiprocessing
from math import ceil
from pymacaron.config import get_config


log = pymlogger(__name__)


# Calculate resources available on this container hardware.
# Used by pymacaron-async, pymacaron-gcp and pymacaron-docker

def get_gunicorn_worker_count(cpu_count=None):
    """Return the number of gunicorn worker to run on this container hardware"""
    if cpu_count:
        return cpu_count * 2 + 1
    return multiprocessing.cpu_count() * 2 + 1


def get_celery_worker_count(cpu_count=None):
    """Return the number of celery workers to run on this container hardware"""
    conf = get_config()
    if hasattr(conf, 'worker_count'):
        # Start worker_count parrallel celery workers
        return conf.worker_count
    if cpu_count:
        return cpu_count * 2
    c = multiprocessing.cpu_count() * 2
    # Minimum worker count == 2
    if c < 2:
        c == 2
    return c


# Memory required, in Mb, by one gunicorn or celery worker:
GUNICORN_WORKER_MEM = 400
CELERY_WORKER_MEM = 200

def get_memory_limit(default_celery_worker_count=None, cpu_count=None):
    """Return the memory in Megabytes required to run pymacaron on this container hardware"""
    # Let's calculate how much memory this pymacaron config requires for 1 container
    celery_count = default_celery_worker_count
    if not celery_count:
        celery_count = get_celery_worker_count(cpu_count=cpu_count)
    return ceil(get_gunicorn_worker_count(cpu_count=cpu_count) * GUNICORN_WORKER_MEM + celery_count * CELERY_WORKER_MEM)


def get_celery_worker_memory_limit():
    return CELERY_WORKER_MEM * 1024
